@@ -57,6 +57,8 @@ class Model(object):
         self._test = test
         self._inflections = {}
         self._references = []
+        self._free_parameters = []
+        self._user_fixed_parameters = []
 
         self._draw_limit_reached = False
 
@@ -359,7 +361,28 @@ class Model(object):
 
         prt.message('loading_data', inline=True)
 
+        # Fix user-specified parameters.
         fixed_parameters = []
+        for task in self._call_stack:
+            for fi, param in enumerate(user_fixed_parameters):
+                if (task == param or
+                        self._call_stack[task].get(
+                            'class', '') == param):
+                    fixed_parameters.append(task)
+                    if fi < len(user_fixed_parameters) - 1 and is_number(
+                            user_fixed_parameters[fi + 1]):
+                        value = float(user_fixed_parameters[fi + 1])
+                        if value not in self._call_stack:
+                            self._call_stack[task]['value'] = value
+                    if 'min_value' in self._call_stack[task]:
+                        del self._call_stack[task]['min_value']
+                    if 'max_value' in self._call_stack[task]:
+                        del self._call_stack[task]['max_value']
+                    self._modules[task].fix_value(
+                        self._call_stack[task]['value'])
+
+        self.determine_free_parameters(fixed_parameters)
+
         for task in self._call_stack:
             cur_task = self._call_stack[task]
             self._modules[task].set_event_name(event_name)
@@ -388,24 +411,8 @@ class Model(object):
             elif cur_task['kind'] == 'sed':
                 self._modules[task].set_data(band_sampling_points)
 
-            # Fix user-specified parameters.
-            for fi, param in enumerate(user_fixed_parameters):
-                if (task == param or
-                        self._call_stack[task].get(
-                            'class', '') == param):
-                    fixed_parameters.append(task)
-                    if fi < len(user_fixed_parameters) - 1 and is_number(
-                            user_fixed_parameters[fi + 1]):
-                        value = float(user_fixed_parameters[fi + 1])
-                        if value not in self._call_stack:
-                            self._call_stack[task]['value'] = value
-                    if 'min_value' in self._call_stack[task]:
-                        del self._call_stack[task]['min_value']
-                    if 'max_value' in self._call_stack[task]:
-                        del self._call_stack[task]['max_value']
-                    self._modules[task].fix_value(
-                        self._call_stack[task]['value'])
-
+        # Determine free parameters again as setting data may have fixed some
+        # more.
         self.determine_free_parameters(fixed_parameters)
 
         self.exchange_requests()
@@ -503,8 +510,11 @@ class Model(object):
         new_call_stack = OrderedDict()
         for task in self._call_stack:
             cur_task = self._call_stack[task]
-            if (cur_task.get('class', '') == 'variance' and
-                    'band' in listify(variance_for_each)):
+            vfe = listify(variance_for_each)
+            if task == 'variance' and 'band' in vfe:
+                vfi = vfe.index('band') + 1
+                mwfd = float(vfe[vfi]) if (vfi < len(vfe) and is_number(
+                    vfe[vfi])) else self.MIN_WAVE_FRAC_DIFF
                 # Find photometry in call stack.
                 for ptask in self._call_stack:
                     if ptask == 'photometry':
@@ -518,9 +528,11 @@ class Model(object):
                 variance_bands = []
                 for bi, (awav, band) in enumerate(band_pairs):
                     wave_frac_diff = abs(awav - owav) / (awav + owav)
-                    if wave_frac_diff < self.MIN_WAVE_FRAC_DIFF:
+                    if wave_frac_diff < mwfd:
                         continue
                     new_task_name = '-'.join([task, 'band', band])
+                    if new_task_name in self._call_stack:
+                        continue
                     new_task = deepcopy(cur_task)
                     new_call_stack[new_task_name] = new_task
                     if 'latex' in new_task:
@@ -570,6 +582,7 @@ class Model(object):
     def determine_free_parameters(self, extra_fixed_parameters=[]):
         """Generate `_free_parameters` and `_num_free_parameters`."""
         self._free_parameters = []
+        self._user_fixed_parameters = []
         self._num_variances = 0
         for task in self._call_stack:
             cur_task = self._call_stack[task]
@@ -581,8 +594,15 @@ class Model(object):
                 self._free_parameters.append(task)
                 if cur_task.get('class', '') == 'variance':
                     self._num_variances += 1
+            elif (cur_task['kind'] == 'parameter'
+                  and task in extra_fixed_parameters):
+                self._user_fixed_parameters.append(task)
         self._num_free_parameters = len(self._free_parameters)
 
+    def is_parameter_fixed_by_user(self, parameter):
+        """Return whether a parameter is fixed by the user."""
+        return parameter in self._user_fixed_parameters
+
     def get_num_free_parameters(self):
         """Return number of free parameters."""
         return self._num_free_parameters
@@ -745,7 +765,7 @@ class Model(object):
                 p = draw
                 score = None
                 break
-            score = self.likelihood(draw)
+            score = self.ln_likelihood(draw)
             if draw_cnt >= self.DRAW_LIMIT and not self._draw_limit_reached:
                 self._printer.message('draw_limit_reached', warning=True)
                 self._draw_limit_reached = True
@@ -798,11 +818,23 @@ class Model(object):
 
     def likelihood(self, x):
         """Return score related to maximum likelihood."""
+        return np.exp(self.ln_likelihood(x))
+
+    def ln_likelihood(self, x):
+        """Return ln(likelihood)."""
         outputs = self.run_stack(x, root='objective')
         return outputs['value']
 
+    def free_parameter_names(self, x):
+        """Return list of free parameter names."""
+        return self._free_parameters
+
     def prior(self, x):
         """Return score related to paramater priors."""
+        return np.exp(self.ln_prior(x))
+
+    def ln_prior(self, x):
+        """Return ln(prior)."""
         prior = 0.0
         for pi, par in enumerate(self._free_parameters):
             lprior = self._modules[par].lnprior_pdf(x[pi])
@@ -815,14 +847,14 @@ class Model(object):
         for key in sorted(kwargs):
             x.append(kwargs[key])
 
-        l = self.likelihood(x) + self.prior(x)
+        l = self.ln_likelihood(x) + self.ln_prior(x)
         if not np.isfinite(l):
             return LOCAL_LIKELIHOOD_FLOOR
         return l
 
     def fprob(self, x):
         """Return score for fracking."""
-        l = -(self.likelihood(x) + self.prior(x))
+        l = -(self.ln_likelihood(x) + self.ln_prior(x))
         if not np.isfinite(l):
             return -LOCAL_LIKELIHOOD_FLOOR
         return l
