@@ -37,15 +37,15 @@ class Printer(object):
 
         BLUE = '\033[0;94m'
         BOLD = '\033[0;1m'
-        CYAN = '\033[0;96m'
+        CYAN = '\033[38;5;6m'
         END = '\033[0m'
         GREEN = '\033[0;92m'
         HEADER = '\033[0;95m'
         MAGENTA = '\033[1;35m'
-        ORANGE = '\033[38;5;214m'
+        ORANGE = '\033[38;5;202m'
         RED = '\033[0;91m'
         UNDERLINE = '\033[4m'
-        YELLOW = '\033[0;93m'
+        YELLOW = '\033[38;5;220m'
 
         codes = {
             '!b': BLUE,
